#ifndef RBTree__H__
#define RBTree__H__

#include <cassert>
#include <ostream>

typedef int KEY;

class RBTree
{
public:
   enum Color
   {
      Red,
      Black
   };

   struct RBNode
   {
      RBNode* parent;
      RBNode* right;
      RBNode* left;
      KEY key;
      Color color;

      RBNode(KEY const& k, RBNode* p, RBNode* r, RBNode* l, Color col);
      RBNode(KEY const& k, RBNode* nil);

      RBNode* find(KEY const& key, RBNode* nil);
      RBNode* min(RBNode* nil);
      RBNode* max(RBNode* nil);
      RBNode* next(RBNode* nil);
      unsigned int height(RBNode* nil) const; 
      unsigned int black_height(RBNode* nil) const; 

      void print(std::ostream& F, RBNode* nil);
      void check_it(RBNode* nil);
   };

   RBTree();
   ~RBTree();

   void insert(KEY const& key);
   void remove(KEY const& key);
   RBNode* find(KEY const& key);
  
   void print(std::ostream& F);
   bool empty() const                       { return root == nil; }
   unsigned int size() const                { return _size; }
   unsigned int height() const              { return root->height(nil); }
   unsigned int black_height() const        { return root->black_height(nil); }


   void right_rotate(RBNode* node);
   void left_rotate(RBNode* node);
   void rb_insert(RBNode* node);
   RBNode* rb_remove(RBNode* node);
   void rb_remove_fixup(RBNode* node);

   void check_it();
   void fix_nil();

private:
   void insert (RBNode* pnode);
   RBTree::RBNode* remove (RBNode* pnode);

   RBNode* root;
   RBNode  _nil;
   RBNode* nil;
   unsigned int _size;
};



// INLINE

inline
RBTree::RBNode::RBNode(KEY const& k, RBNode* p, RBNode* r, RBNode* l, Color col)
: parent(p), right(r), left(l), key(k), color(col)
{ }

inline
RBTree::RBNode::RBNode(KEY const& k, RBNode* nil)
: parent(nil), right(nil), left(nil), key(k), color(Red)
{ }

inline
RBTree::RBTree()
: root(&_nil), _nil(0, &_nil, &_nil, &_nil, Black), nil(&_nil), _size(0)
{ }

inline void 
RBTree::remove(KEY const& key)
{
   rb_remove(find(key));
}



#endif